'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _minimist = require('minimist');

var _minimist2 = _interopRequireDefault(_minimist);

var _colors = require('colors');

var _colors2 = _interopRequireDefault(_colors);

var _colors3 = require('pad/lib/colors');

var _colors4 = _interopRequireDefault(_colors3);

var _progress = require('progress');

var _progress2 = _interopRequireDefault(_progress);

var _quizGenerator = require('../quiz-generator');

var _quizGenerator2 = _interopRequireDefault(_quizGenerator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; } /* eslint-disable no-console */


const showUsage = () => {
  console.log(`word-quiz-generator coverage --help
word-quiz-generator coverage --material=<path> --sources=<paths> [--lang]
                             [--show-details] [--show-lemma]
                             [--sentenceSeparator=<RegExp>] [--clauseRegExp=<RegExp>]
                             [--wordRegExp=<RegExp>] [--wordBoundaryRegExp=<RegExp>]
                             [--abbrRegExp=<RegExp>]

Measure the coverage of words/phrases in the given material against the given sources.

-h, --help
    Show this usage.
-m, --material=<path>
    Path to a material.
-s, --sources=<paths>
    Comma-separated path strings to sources.
-l, --lang=<lang>
    IETF langage tag in which the material are written.
    This determines how to extract a word/phrase or sentence from a text.
    If you need more precise control over the extraction algorithm,
    please use '--sentenceSeparator', '--clauseRegExp', '--wordRegExp',
    '--wordBoundaryRegExp', and/or '--abbrRegExp' to override.
    Default: 'en' (English)
-d, --show-details
    Show additional information such as the list of uncovered words/phrases.
--show-lemma
    Show lemmas of each phrase when a phrase and its lemma are different each other.
    This option helps you to detect phrases registered as non-lemma form,
    which may not be able to be found by the quiz generator and make the coverage lower.
--sentenceSeparator=<RegExp>
    Regular expression representing a sentence separator.
--clauseRegExp=<RegExp>
    Regular expression representing a clause.
--wordRegExp=<RegExp>
    Regular expression representing a word.
--wordBoundaryRegExp=<RegExp>
    Regular expression representing a word boundary.
--abbrRegExp=<RegExp>
    Regular expression representing an abbreviation mark.`);
};

exports.default = (() => {
  var _ref = _asyncToGenerator(function* (args) {
    const argv = (0, _minimist2.default)(args, {
      string: ['material', 'sources', 'lang', 'sentenceSeparator', 'clauseRegExp', 'wordRegExp', 'wordBoundaryRegExp', 'abbrRegExp'],
      boolean: ['show-details', 'show-lemma', 'help'],
      alias: {
        m: 'material',
        s: 'sources',
        l: 'lang',
        d: 'show-details',
        h: 'help'
      },
      default: {
        lang: 'en'
      }
    });

    if (argv.help) {
      showUsage();
      process.exit(0);
    }

    if (!argv.material || !argv.sources) {
      showUsage();
      process.exit(1);
    }

    const material = argv.material;
    const sources = argv.sources;
    const lang = argv.lang;
    const sentenceSeparator = argv.sentenceSeparator && new RegExp(argv.sentenceSeparator, 'g');
    const clauseRegExp = argv.clauseRegExp && new RegExp(argv.clauseRegExp, 'g');
    const wordRegExp = argv.wordRegExp && new RegExp(argv.wordRegExp, 'g');
    const wordBoundaryRegExp = argv.wordBoundaryRegExp && new RegExp(argv.wordBoundaryRegExp);
    const abbrRegExp = argv.abbrRegExp && new RegExp(argv.abbrRegExp, 'g');
    const generator = new _quizGenerator2.default({
      material,
      sources,
      lang,
      sentenceSeparator,
      clauseRegExp,
      wordRegExp,
      wordBoundaryRegExp,
      abbrRegExp
    });

    yield generator.init();

    const phrases = generator._phrases;
    const total = phrases.length;
    const uncovered = [];
    const progress = new _progress2.default('[:bar] :percent :etas', {
      complete: '=',
      incomplete: ' ',
      width: 20,
      total: phrases.length
    });

    const promises = phrases.map(function (phrase) {
      return _asyncToGenerator(function* () {
        const isCovered = !!(yield generator.question(phrase));

        if (!isCovered) {
          uncovered.push(phrase);
        }

        progress.tick();
      })();
    });

    yield Promise.all(promises);

    const coverage = (total - uncovered.length) * 100 / total;

    console.log(_colors2.default.bold(`Coverage: ${ coverage.toFixed(2) }%`));

    if (argv['show-details']) {
      console.log(_colors2.default.bold('\nUncovered Phrases'));

      console.log(`
-----
Legend: ${ '#'.grey } phrase ${ 'answer'.grey } ${ 'reference'.green } ${ 'lemmas'.red }

- A reference name is shown if a phrase is found in an original version of source,
  which means you may somehow make the phrase detectable from the generator,
  such as by turning them into their lemmas.

- Lemmas of a phrase are shown if they are different from an original phrase,
  which indicates the phrase is shown here because they are stored in natural form
  and not recognizable to the generator.
  ${ 'Lemmas are shown only when you run with \'--show-lemma\' option.'.bold }
-----
`);

      // eslint-disable-next-line global-require, import/no-dynamic-require
      const lemmatizer = require(`../lemmatizers/${ argv.lang }.js`).default;
      const lookUpOriginalSources = function (phrase) {
        return new Promise(function (resolve) {
          Promise.all(generator._sources.map(function (source) {
            return _asyncToGenerator(function* () {
              const text = yield source.getText();

              if (phrase.split('|').some(function (exp) {
                return text.includes(exp);
              })) {
                resolve(source);
              }
            })();
          })).then(function () {
            return resolve(null);
          });
        });
      };

      uncovered.forEach(function (phrase) {
        return _asyncToGenerator(function* () {
          const [reference, lemma] = yield Promise.all([_asyncToGenerator(function* () {
            const source = yield lookUpOriginalSources(phrase.phrase);
            const ref = source ? yield source.getReference() : '';

            return ref;
          })(), _asyncToGenerator(function* () {
            if (!argv['show-lemma']) {
              return Promise.resolve('');
            }

            return lemmatizer(phrase.phrase);
          })()]);

          const message = [];

          message.push((0, _colors4.default)(2, _colors2.default.grey(phrase.section)));
          message.push((0, _colors4.default)(phrase.phrase, 20));
          message.push((0, _colors4.default)(_colors2.default.grey(phrase.answer), 20));
          message.push((0, _colors4.default)(_colors2.default.green(reference), 10));
          message.push(_colors2.default.red(lemma !== phrase.phrase ? lemma : ''));

          console.log(message.join(' '));
        })();
      });
    }
  });

  return function (_x) {
    return _ref.apply(this, arguments);
  };
})();
//# sourceMappingURL=coverage.js.map