'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _minimist = require('minimist');

var _minimist2 = _interopRequireDefault(_minimist);

var _colors = require('colors');

var _colors2 = _interopRequireDefault(_colors);

var _quizGenerator = require('../quiz-generator');

var _quizGenerator2 = _interopRequireDefault(_quizGenerator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; } /* eslint-disable no-console */


const showUsage = () => {
  console.log(`word-quiz-generator generate --help
word-quiz-generator generate --material=<path> --sources=<paths> --sections --size
                             [--instruction] [--lang] [--sentenceSeparator=<RegExp>]
                             [--clauseRegExp=<RegExp>] [--wordRegExp=<RegExp>]
                             [--wordBoundaryRegExp=<RegExp>] [--abbrRegExp=<RegExp>]

Generate a quiz and put it to stdout using the given material and sources.

-h, --help
    Show this usage.
-m, --material=<path>
    Path to a material.
-s, --sources=<paths>
    Comma-separated path strings to sources.
-e, --sections=<sections>
    Quiz coverage. e.g. '1-10', '5', '3-'
-i, --size=<num>
    The number of questions.
-l, --lang=<lang>
    IETF langage tag in which the material are written.
    This determines how to extract a word/phrase or sentence from a text.
    If you need more precise control over the extraction algorithm,
    please use '--sentenceSeparator', '--clauseRegExp', '--wordRegExp',
    '--wordBoundaryRegExp', and/or '--abbrRegExp' to override.
    Default: 'en' (English)
--instruction
    The instruction text located at top of the quiz.
    Default: 'Write down the meaning of underlined words/phrases.'
--sentenceSeparator=<RegExp>
    Regular expression representing a sentence separator.
--clauseRegExp=<RegExp>
    Regular expression representing a clause.
--wordRegExp=<RegExp>
    Regular expression representing a word.
--wordBoundaryRegExp=<RegExp>
    Regular expression representing a word boundary.
--abbrRegExp=<RegExp>
    Regular expression representing an abbreviation mark.`);
};

exports.default = (() => {
  var _ref = _asyncToGenerator(function* (args) {
    const argv = (0, _minimist2.default)(args, {
      string: ['material', 'sources', 'sections', 'size', 'instruction', 'lang', 'sentenceSeparator', 'clauseRegExp', 'wordRegExp', 'wordBoundaryRegExp', 'abbrRegExp'],
      boolean: ['help'],
      alias: {
        m: 'material',
        s: 'sources',
        l: 'lang',
        e: 'sections',
        i: 'size'
      },
      default: {
        instruction: 'Write down the meaning of underlined words/phrases.'
      }
    });

    if (argv.help) {
      showUsage();
      process.exit(0);
    }

    if (!argv.material || !argv.sources || !argv.sections || !argv.size) {
      showUsage();
      process.exit(1);
    }

    const material = argv.material;
    const sources = argv.sources;
    const lang = argv.lang;
    const sentenceSeparator = argv.sentenceSeparator && new RegExp(argv.sentenceSeparator, 'g');
    const clauseRegExp = argv.clauseRegExp && new RegExp(argv.clauseRegExp, 'g');
    const wordRegExp = argv.wordRegExp && new RegExp(argv.wordRegExp, 'g');
    const wordBoundaryRegExp = argv.wordBoundaryRegExp && new RegExp(argv.wordBoundaryRegExp);
    const abbrRegExp = argv.abbrRegExp && new RegExp(argv.abbrRegExp, 'g');

    const generator = new _quizGenerator2.default({
      material,
      sources,
      lang,
      sentenceSeparator,
      clauseRegExp,
      wordRegExp,
      wordBoundaryRegExp,
      abbrRegExp
    });
    yield generator.init();

    const sections = argv.sections;
    const size = Number(argv.size);

    // generate
    const questions = yield generator.quiz({ sections, size });

    // instruction
    console.log(_colors2.default.bold(argv.instruction));

    // questions
    questions.forEach(function (question, i) {
      const index = i + 1;
      const decoratedSentence = [];
      const { body, reference } = question;

      // Set underlines to question parts.
      // eslint-disable-next-line no-cond-assign
      body.forEach(function ({ text, isQuestionPart }) {
        if (isQuestionPart) {
          decoratedSentence.push(_colors2.default.underline(text));
        } else {
          decoratedSentence.push(text);
        }
      });

      console.log(`(${ index })\t${ decoratedSentence.join('') }\t(${ reference })`);
    });

    // title in answers section
    console.log(_colors2.default.bold('\nAnswer Keys'));

    // answers
    questions.forEach(function (q, i) {
      console.log(`(${ i + 1 })\t${ q.answer }`);
    });
  });

  return function (_x) {
    return _ref.apply(this, arguments);
  };
})();
//# sourceMappingURL=generate.js.map