'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

let clean = (() => {
  var _ref = _asyncToGenerator(function* (dir) {
    const files = yield (0, _utils.fetchFileList)(dir, /\.(?:preprocessed|lemmatized)$/);
    yield Promise.all(files.map(function (_path) {
      return _fsExtraPromise2.default.removeAsync(_path);
    }));
  });

  return function clean(_x) {
    return _ref.apply(this, arguments);
  };
})();

var _path2 = require('path');

var _path3 = _interopRequireDefault(_path2);

var _minimist = require('minimist');

var _minimist2 = _interopRequireDefault(_minimist);

var _fsExtraPromise = require('fs-extra-promise');

var _fsExtraPromise2 = _interopRequireDefault(_fsExtraPromise);

var _source = require('../source');

var _source2 = _interopRequireDefault(_source);

var _utils = require('../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; } /* eslint-disable no-console, global-require, import/no-dynamic-require */


const showUsage = () => {
  console.log(`word-quiz-generator make --help
word-quiz-generator make --src=<path> --lang=<lang>
word-quiz-generator make --src=<path> [--preprocessor=<path>] [--lemmatizer=<path>]

Generate preprocessed and lemmatized texts from the given sources.

-h, --help
    Show this usage.
-s, --src=<paths>
    Comma-separated path strings to be processed.
-l, --lang=<lang>
    IETF langage tag in which source texts are written.
    This determines which built-in preprocesser and lemmatizer should be used.
    If you want to use your custom ones, please use '--preprocessor' and '--lemmatizer' options.
    Default: 'en' (English)
--preprocessor=<path>
  Path to a custom preprocessor.
--lemmatizer=<path>
  Path to a custom lemmatizer.`);
};

function getPreprocessor(argv) {
  try {
    if (argv.preprocessor) {
      const preprocessor = require(argv.preprocessor);
      return preprocessor.default || preprocessor;
    } else if (argv.lang) {
      return require(`../preprocessors/${ argv.lang }.js`).default;
    }
  } catch (err) {
    console.error('Unable to load the preprocessor: %c', err.message);
  }

  return null;
}

function getLemmatizer(argv) {
  try {
    if (argv.lemmatizer) {
      const lemmatizer = require(argv.lemmatizer);
      return lemmatizer.default || lemmatizer;
    } else if (argv.lang) {
      return require(`../lemmatizers/${ argv.lang }.js`).default;
    }
  } catch (err) {
    console.error('Unable to load the lemmatizer: %c', err.message);
  }

  return null;
}

exports.default = (() => {
  var _ref2 = _asyncToGenerator(function* (args) {
    const argv = (0, _minimist2.default)(args, {
      string: ['src', 'lang', 'preprocessor', 'lemmatizer'],
      boolean: ['help'],
      alias: {
        s: 'src',
        l: 'lang',
        h: 'help'
      },
      default: {
        lang: 'en',
        help: false
      }
    });

    if (argv.help) {
      showUsage();
      process.exit(0);
    }

    if (!argv.src) {
      console.error('ERR: No src is specified.');
      showUsage();
      process.exit(1);
    }

    console.log(`Src: ${ _path3.default.resolve(argv.src) }`);

    yield clean(argv.src);

    const preprocessor = getPreprocessor(argv);
    const lemmatizer = getLemmatizer(argv);
    const files = yield (0, _utils.fetchFileList)(argv.src, /\.txt$/);

    files.forEach(function (_path) {
      return _asyncToGenerator(function* () {
        try {
          const source = new _source2.default(_path);

          if (preprocessor) {
            yield source.preprocess(preprocessor);
            console.log(`Finish preprocessing: ${ _path }`);
          }

          if (lemmatizer) {
            yield source.lemmatize(lemmatizer);
            console.log(`Finish lemmatizing: ${ _path }`);
          }
        } catch (ex) {
          console.error(ex.stack);
        }
      })();
    });
  });

  return function (_x2) {
    return _ref2.apply(this, arguments);
  };
})();
//# sourceMappingURL=make.js.map