'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _child_process = require('child_process');

var _shellQuote = require('shell-quote');

var _shellQuote2 = _interopRequireDefault(_shellQuote);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Customizable MeCab connector.
 */
class MeCab {
  /**
   * @param {string} command a command string to be executed via spawn.
   */
  constructor({ command = 'mecab' }) {
    this._command = command;
  }

  _getCommandStr(text) {
    const echoCommand = _shellQuote2.default.quote(['echo', text]);
    return `${ echoCommand } | ${ this._command }`;
  }

  /**
   * Analyze a given text with MeCab.
   * @param {string} text a text string to be parsed.
   * @return {Promise<Array<Array<string>>}
   */
  parse(text) {
    return new Promise((resolve, reject) => {
      (0, _child_process.exec)(this._getCommandStr(text), (err, result) => {
        /* istanbul ignore if */
        if (err) {
          return reject(err);
        }

        const parsed = result.split('\n').map(line => line.replace(/\t/g, ',').split(',')).filter(line => line.length >= 6);

        return resolve(parsed);
      });
    });
  }
}
exports.default = MeCab;
//# sourceMappingURL=mecab.js.map