'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _stanfordCorenlp = require('stanford-corenlp');

var _stanfordCorenlp2 = _interopRequireDefault(_stanfordCorenlp);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

// Installing stanford-corenlp is not necessary if you won't use this module,
// so it is not listed in package.json.
// eslint-disable-next-line import/no-extraneous-dependencies


const coreNLP = new _stanfordCorenlp2.default.StanfordNLP({
  nlpPath: _path2.default.join(__dirname, '../../vendor/corenlp/corenlp'),
  version: '3.5.2',
  annotators: ['tokenize', 'ssplit']
});

function processWithCoreNLP(text) {
  return new Promise((resolve, reject) => {
    coreNLP.process(text, (err, result) => {
      /* istanbul ignore if */
      if (err) {
        return reject(err);
      }
      return resolve(result);
    });
  });
}

exports.default = (() => {
  var _ref = _asyncToGenerator(function* (text) {
    const emended = text
    // Remove unnecessary line breaks.
    .replace(/[\n\r]+/g, ' ')

    // Shorten repeating whitespaces.
    .replace(/ +/g, ' ')

    // Remove soft hyphens.
    .replace(/(\w)-\s+(\w)/g, '$1$2')

    // Replace some full-width signs with their half-width ones.
    .replace(/(?:‘|’)/g, '\'').replace(/(?:“|”)/g, '"').replace(/—/g, '--');

    const result = yield processWithCoreNLP(emended);

    // there is just one sentence and don't need to split it!
    if (!Array.isArray(result.document.sentences.sentence)) {
      return emended;
    }

    const sentences = result.document.sentences.sentence.map(function (sentenceData) {
      const coreNLPToken = sentenceData.tokens.token;
      const tokens = Array.isArray(coreNLPToken) ? coreNLPToken : [coreNLPToken];
      const begin = _lodash2.default.first(tokens).CharacterOffsetBegin;
      const end = _lodash2.default.last(tokens).CharacterOffsetEnd;

      return emended.slice(begin, end);
    });

    return sentences.join('\n');
  });

  return function (_x) {
    return _ref.apply(this, arguments);
  };
})();
//# sourceMappingURL=en.js.map