'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fetchFileList = exports.parseMaterial = undefined;

var _fsExtraPromise = require('fs-extra-promise');

var _fsExtraPromise2 = _interopRequireDefault(_fsExtraPromise);

var _csv = require('csv');

var _csv2 = _interopRequireDefault(_csv);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _flatten = require('lodash/flatten');

var _flatten2 = _interopRequireDefault(_flatten);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

/**
 * Parse a material (CSV-formatted phrase list)
 * and return an array of Phrase object.
 * @param {string} csvPath Path to a CSV file.
 * @return {Promise<Array<Phrase>>}
 */
const parseMaterial = exports.parseMaterial = materialPath => new Promise((resolve, reject) => {
  const input = _fsExtraPromise2.default.createReadStream(materialPath);
  const parser = _csv2.default.parse();
  const transformer = _csv2.default.transform(record => {
    const [section, phrase, answer, ..._] = record;
    return phrase ? { section: Number(section), phrase, answer, _ } : null;
  });
  const phraseList = [];

  input.pipe(parser).pipe(transformer).on('data', row => phraseList.push(row)).on('end', () => resolve(phraseList)).on('error', err => reject(err));
});

/**
 * Fetch a list of all the files and directories
 * under the given comma-separated paths.
 * @param {string} paths comma-separated path strings
 * @param {RegExp} [pattern = /(?:)/] filtering regular expression
 * @return {Promise<Array<string>>}
 */
const fetchFileList = exports.fetchFileList = (paths, pattern = /(?:)/) => Promise.all(paths.split(',').map(p => _asyncToGenerator(function* () {
  const ap = _path2.default.resolve(p);
  const stat = yield _fsExtraPromise2.default.statAsync(ap);

  if (stat.isDirectory()) {
    const ls = yield _fsExtraPromise2.default.readdirAsync(ap);
    const arg = ls.map(function (name) {
      return _path2.default.join(ap, name);
    }).join(',');
    return fetchFileList(arg, pattern);
  }

  if (pattern.test(ap)) {
    return _path2.default.resolve(ap);
  }

  return [];
})())).then(lists => (0, _flatten2.default)(lists));
//# sourceMappingURL=utils.js.map